// @ts-check

/**
 * Create gitignore base file
 */
function gitignoreCreate(callback) {
  const paths = require("../paths");
  const { taskIfDevAndMissingFile } = require("../utils");

  return taskIfDevAndMissingFile(
    paths.join(paths.root, ".gitignore"),
    callback,
    () => {
      const gulp = require("gulp");
      const rename = require("gulp-rename");

      return gulp
        .src(paths.join(paths.self.templates, "gitignore"))
        .pipe(rename(".gitignore"))
        .pipe(gulp.dest(paths.root));
    }
  );
}

/**
 * Create gitignore files for Laravel folders
 *
 * @see https://github.com/laravel/laravel
 */
function gitignoreFoldersCreate() {
  const gulp = require("gulp");
  const rename = require("gulp-rename");
  const { self, root, join } = require("../paths");

  return gulp
    .src(join(self.templates, "gitignore-folder"))
    .pipe(rename(".gitignore"))
    .pipe(gulp.dest(join(root, "storage/app")))
    .pipe(gulp.dest(join(root, "storage/app/public")))
    .pipe(gulp.dest(join(root, "storage/debugbar")))
    .pipe(gulp.dest(join(root, "storage/framework")))
    .pipe(gulp.dest(join(root, "storage/framework/cache")))
    .pipe(gulp.dest(join(root, "storage/framework/sessions")))
    .pipe(gulp.dest(join(root, "storage/framework/testing")))
    .pipe(gulp.dest(join(root, "storage/framework/views")))
    .pipe(gulp.dest(join(root, "storage/logs")));
}

/**
 * Collect subtasks in a single task that respect the gulp async lifecycle by
 * calling the callback on completion.
 */
module.exports = function gitignore(callback) {
  const gulp = require("gulp");
  return gulp.parallel(gitignoreCreate, gitignoreFoldersCreate)(callback);
};
